! At the moment this is a hard-coded file and not extracted from
! omegalib.nw
!
! Copyright (C) 2000-2004 by Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
! and others
!
! O'Mega is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! O'Mega is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
module omega_parameters_simplest
  use omega_kinds
  use omega_constants
  implicit none
  private
  public :: setup_parameters, print_parameters
  real(kind=omega_prec), dimension(45), save, public :: mass = 0, width = 0
  real(kind=omega_prec), parameter, public :: GeV = 1.0_double
  real(kind=omega_prec), parameter, public :: MeV = GeV / 1000
  real(kind=omega_prec), parameter, public :: keV = MeV / 1000
  real(kind=omega_prec), parameter, public :: TeV = GeV * 1000
  real(kind=omega_prec), save, public :: &
       alpha = 1.0_double / 137.0359895_double, &
       sin2thw = 0.23124_double
  complex(kind=omega_prec), save, private :: vev
  complex(kind=omega_prec), save, public :: &
       qlep = 0, qup = 0, qdwn = 0, gcc = 0, qw = 0, &
       gzww = 0, gwww = 0, ghww = 0, ghhww = 0, ghzz = 0, ghhzz = 0, &
       ghbb = 0, ghtt = 0, ghcc = 0, ghtautau = 0, gh3 = 0, gh4 = 0, &
       ghgaga = 0, ghgaz = 0, &
       iqw = 0, igzww = 0, igwww = 0, &
       gw4 = 0, gzzww = 0, gazww = 0, gaaww = 0, &
       gs = 0, igs = 0
  real(kind=omega_prec), save, public :: xia = 1, xi0 = 1, xipm = 1
  complex(kind=omega_prec), dimension(2), save, public :: &
       gnclep = 0, gncneu = 0, gncup = 0, gncdwn = 0
  complex(kind=omega_prec), save, public :: &
       fudge_o1 = 1, fudge_o2 = 1, fudge_o3 = 1, fudge_o4 = 1
  complex(kind=omega_prec), save, public :: &
       ghmumu = 0
  complex(kind=omega_prec), save, public :: &
       ghaa = 0, ghgg = 0, geaa = 0, geaz = 0, gegg = 0
  complex(kind=omega_prec), save, public :: &
       gh0ww = 0, gh0zz = 0, &
       gh0tt = 0, gh0bb = 0, gh0cc = 0, gh0tautau = 0, gh0mumu = 0, &
       gahh = 0, gzhh = 0
  complex(kind=omega_prec), dimension(2), save, public :: &
       ghptb = 0, ghpcs = 0, ghptaunu = 0, ghpmunu = 0
contains
  subroutine setup_parameters ()
    real(kind=omega_prec) :: e, g, sinthw, costhw, qelep, qeup, qedwn
    mass(1) =   5.0 * MeV
    mass(2) =   3.0 * MeV
    mass(3) = 100.0 * MeV
    mass(4) =   1.2 * GeV
    mass(5) =   4.2 * GeV
    mass(6) = 174.0 * GeV
    width(1:5) = 0
    width(6) =  1.3 * GeV
    mass(11) =    0.51099907 * MeV
    mass(12) =    0
    mass(13) =  105.658389 * MeV
    mass(14) =    0
    mass(15) = 1777.05 * MeV
    mass(16) =    0
    width(11:16) = 0
    mass(21) =  0
    mass(22) =  0
    width(21:22) = 0
    mass(23) =  91.187 * GeV
    width(23) =  2.490 * GeV
    mass(24) =  80.41 * GeV
    width(24) =  2.06 * GeV
    mass(25) = 120.00 * GeV
    width(25) =  5.00 * GeV
    mass(35) = 10000 * GeV
    width(35) =  0
    mass(26) =  xi0 * mass(23)
    width(26) =  0
    mass(27) =  xipm * mass(24)
    width(27) =  0
    e = sqrt (4 * PI * alpha)
    qelep = - 1
    qeup = 2.0_omega_prec / 3.0_omega_prec
    qedwn = - 1.0_omega_prec / 3.0_omega_prec
    sinthw = sqrt (sin2thw)
    costhw = sqrt (1 - sin2thw)
    g = e / sinthw
    gcc = - g / 2 / sqrt (2.0_double)
    gncneu(1) = - g / 2 / costhw * ( + 0.5_double)
    gnclep(1) = - g / 2 / costhw * ( - 0.5_double - 2 * qelep * sin2thw)
    gncup(1)  = - g / 2 / costhw * ( + 0.5_double - 2 * qeup  * sin2thw)
    gncdwn(1) = - g / 2 / costhw * ( - 0.5_double - 2 * qedwn * sin2thw)
    gncneu(2) = - g / 2 / costhw * ( + 0.5_double)
    gnclep(2) = - g / 2 / costhw * ( - 0.5_double)
    gncup(2)  = - g / 2 / costhw * ( + 0.5_double)
    gncdwn(2) = - g / 2 / costhw * ( - 0.5_double)
    qlep = - e * qelep
    qup = - e * qeup
    qdwn = - e * qedwn
    qw = e
    iqw = (0,1)*qw
    gzww = g * costhw
    igzww = (0,1)*gzww
    gwww = g
    igwww = (0,1)*gwww
    ghww = mass(24) * g
    ghhww = g**2 / 2.0_omega_prec
    ghzz = mass(23) * g / costhw
    ghhzz = g**2 / 2.0_omega_prec / costhw**2
    gw4 = g**2
    gzzww = gzww**2
    gazww = gzww*e
    gaaww = e**2
    vev = 2.0 * mass(24) / g
    ghtt = - mass(6) / vev
    ghbb = - mass(5) / vev
    ghcc = - mass(4) / vev
    ghtautau = - mass(15) / vev
    gh3 = - 3 * mass(25)**2 / vev
    gh4 = - 3 * mass(25)**2 / vev**2
  end subroutine setup_parameters
  subroutine print_parameters ()
    print *, "Quark masses:"
    print *, mass(2:6:2)
    print *, mass(1:5:2)
    print *, "Lepton masses:"
    print *, mass(12:16:2)
    print *, mass(11:15:2)
    print *, "Quark widths:"
    print *, width(2:6:2)
    print *, width(1:5:2)
    print *, "Lepton widths:"
    print *, width(12:16:2)
    print *, width(11:15:2)
    print *, "SU(2)xU(1) Gauge boson masses/widths:"
    print *, mass(22:24)
    print *, width(22:24)
    print *, "Higgs boson and gluon masses/widths:"
    print *, mass(25), mass(21)
    print *, width(25), width(21) 
    print *, "Neutral current couplings:"
    print *, "U:", gncup
    print *, "D:", gncdwn
    print *, "N:", gncneu
    print *, "L:", gnclep
    print *, "Fermion charges:"
    print *, "U:", qup
    print *, "D:", qdwn
    print *, "L:", qlep
    print *, "TGC:"
    print *, "WWA:", iqw
    print *, "WWZ:", igzww
    print *, "WWW:", igwww
    print *, "WWH:", ghww
    print *, "WWHH:", ghhww
    print *, "ZZHH:", ghhzz       
  end subroutine print_parameters
end module omega_parameters_simplest
